﻿using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.EntityFrameworkCore;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using System.Linq;
using System.Reflection;

using IdentityServer4.EntityFramework.DbContexts;
using IdentityServer4.EntityFramework.Mappers;
using IdentityServer4.Models;

using TestIdentityServer.Models;

namespace TestIdentityServer
{
    /// <summary>
    /// 시작
    /// </summary>
    public class Startup
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 구성 - Configuration

        /// <summary>
        /// 구성
        /// </summary>
        public IConfiguration Configuration { get; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - Startup(configuration)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="configuration">구성</param>
        public Startup(IConfiguration configuration)
        {
            Configuration = configuration;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 서비스 컬렉션 구성하기 - ConfigureServices(services)

        /// <summary>
        /// 서비스 컬렉션 구성하기
        /// </summary>
        /// <param name="services">서비스 컬렉션</param>
        public void ConfigureServices(IServiceCollection services)
        {
            string connectionString   = Configuration.GetConnectionString("DefaultConnection");
            string migrationsAssembly = typeof(Startup).GetTypeInfo().Assembly.GetName().Name;

            services.AddIdentityServer()
                .AddTestUsers(TestUserData.UserList)
                .AddDeveloperSigningCredential()
                .AddConfigurationStore
                (
                    options =>
                    {
                        options.ConfigureDbContext = builder => builder.UseSqlServer
                        (
                            connectionString,
                            sqlServerBuilder => sqlServerBuilder.MigrationsAssembly(migrationsAssembly)
                        );
                    }
                )
                .AddOperationalStore
                (
                    options =>
                    {
                        options.ConfigureDbContext = builder => builder.UseSqlServer
                        (
                            connectionString,
                            sqlServerBuilder => sqlServerBuilder.MigrationsAssembly(migrationsAssembly)
                        );
                    }
                );

            services.AddControllersWithViews();
        }

        #endregion
        #region 구성하기 - Configure(app, environment)

        /// <summary>
        /// 구성하기
        /// </summary>
        /// <param name="app">애플리케이션 빌더</param>
        /// <param name="environment">웹 호스트 환경</param>
        public void Configure(IApplicationBuilder app, IWebHostEnvironment environment)
        {
            InitializeDatabase(app);

            if(environment.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            app.UseStaticFiles();

            app.UseRouting();
            
            app.UseIdentityServer();

            app.UseAuthorization();

            app.UseEndpoints
            (
                endpoints =>
                {
                    endpoints.MapControllerRoute
                    (
                        name    : "default",
                        pattern : "{controller=Home}/{action=Index}/{id?}"
                    );
                }
            );
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 데이터베이스 초기화하기 - InitializeDatabase(app)

        /// <summary>
        /// 데이터베이스 초기화하기
        /// </summary>
        /// <param name="app">애플리케이션 빌더</param>
        private void InitializeDatabase(IApplicationBuilder app)
        {
            IServiceScopeFactory serviceScopeFactory = app.ApplicationServices.GetService<IServiceScopeFactory>();

            using(IServiceScope serviceScope = serviceScopeFactory.CreateScope())
            {
                serviceScope.ServiceProvider.GetRequiredService<PersistedGrantDbContext>().Database.Migrate();

                ConfigurationDbContext context = serviceScope.ServiceProvider.GetRequiredService<ConfigurationDbContext>();

                context.Database.Migrate();

                if(!context.Clients.Any())
                {
                    foreach(Client client in TestIdentityServer.Configuration.GetClientList())
                    {
                        context.Clients.Add(client.ToEntity());
                    }

                    context.SaveChanges();
                }

                if(!context.IdentityResources.Any())
                {
                    foreach(IdentityResource resource in TestIdentityServer.Configuration.GetIdentityResourceList())
                    {
                        context.IdentityResources.Add(resource.ToEntity());
                    }

                    context.SaveChanges();
                }

                if(!context.ApiScopes.Any())
                {
                    foreach(ApiScope resource in TestIdentityServer.Configuration.GetAPIScopeList())
                    {
                        context.ApiScopes.Add(resource.ToEntity());
                    }

                    context.SaveChanges();
                }
            }
        }

        #endregion
    }
}